function _define_property(obj, key, value) {
    if (key in obj) {
        Object.defineProperty(obj, key, {
            value: value,
            enumerable: true,
            configurable: true,
            writable: true
        });
    }
    else {
        obj[key] = value;
    }
    return obj;
}
import './global-types.js';
import './schema-builder.js';
import SchemaBuilder, { BasePlugin, createInputValueMapper, mapInputFields, unwrapInputFieldType } from '@pothos/core';
export * from './types.js';
const pluginName = "prismaUtils";
export default pluginName;
function normalizeInputObject(object, nullableFields) {
    if (!object) {
        return object;
    }
    if (typeof object !== "object") {
        return object;
    }
    if (Array.isArray(object)) {
        return object.map((o) => normalizeInputObject(o, nullableFields));
    }
    const mapped = {};
    for (const key of Object.keys(object)) {
        mapped[key] = !nullableFields.has(key) && object[key] === null ? undefined : object[key];
    }
    return mapped;
}
export class PothosPrismaUtilsPlugin extends BasePlugin {
    onOutputFieldConfig(fieldConfig) {
        const argMappings = mapInputFields(fieldConfig.args, this.buildCache, (inputField) => {
            var _inputType_extensions;
            const inputType = this.buildCache.getTypeConfig(unwrapInputFieldType(inputField.type));
            if ((_inputType_extensions = inputType.extensions) === null || _inputType_extensions === void 0 ? void 0 : _inputType_extensions.pothosPrismaInput) {
                var _inputType_extensions1, _inputType_extensions2;
                return typeof ((_inputType_extensions1 = inputType.extensions) === null || _inputType_extensions1 === void 0 ? void 0 : _inputType_extensions1.pothosPrismaInput) === "object" ? (_inputType_extensions2 = inputType.extensions) === null || _inputType_extensions2 === void 0 ? void 0 : _inputType_extensions2.pothosPrismaInput : {
                    nullableFields: new Set()
                };
            }
            return null;
        }, this.mappingCache);
        if (!argMappings) {
            return fieldConfig;
        }
        const argMapper = createInputValueMapper(argMappings, (inputObject, mapping) => {
            var _mapping_value;
            var _mapping_value_nullableFields;
            return normalizeInputObject(inputObject, (_mapping_value_nullableFields = (_mapping_value = mapping.value) === null || _mapping_value === void 0 ? void 0 : _mapping_value.nullableFields) !== null && _mapping_value_nullableFields !== void 0 ? _mapping_value_nullableFields : new Set());
        });
        var _fieldConfig_argMappers;
        return {
            ...fieldConfig,
            argMappers: [
                ...(_fieldConfig_argMappers = fieldConfig.argMappers) !== null && _fieldConfig_argMappers !== void 0 ? _fieldConfig_argMappers : [],
                (args) => argMapper(args)
            ]
        };
    }
    constructor(cache) {
        super(cache, pluginName), _define_property(this, "mappingCache", new Map());
    }
}
SchemaBuilder.registerPlugin(pluginName, PothosPrismaUtilsPlugin);
//# sourceMappingURL=index.js.map
