"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
Object.defineProperty(exports, "schemaBuilder", {
    enumerable: true,
    get: function() {
        return schemaBuilder;
    }
});
const _core = /*#__PURE__*/ _interop_require_wildcard(require("@pothos/core"));
const _pluginprisma = require("@pothos/plugin-prisma");
function _getRequireWildcardCache(nodeInterop) {
    if (typeof WeakMap !== "function") return null;
    var cacheBabelInterop = new WeakMap();
    var cacheNodeInterop = new WeakMap();
    return (_getRequireWildcardCache = function(nodeInterop) {
        return nodeInterop ? cacheNodeInterop : cacheBabelInterop;
    })(nodeInterop);
}
function _interop_require_wildcard(obj, nodeInterop) {
    if (!nodeInterop && obj && obj.__esModule) {
        return obj;
    }
    if (obj === null || typeof obj !== "object" && typeof obj !== "function") {
        return {
            default: obj
        };
    }
    var cache = _getRequireWildcardCache(nodeInterop);
    if (cache && cache.has(obj)) {
        return cache.get(obj);
    }
    var newObj = {
        __proto__: null
    };
    var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor;
    for(var key in obj){
        if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) {
            var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null;
            if (desc && (desc.get || desc.set)) {
                Object.defineProperty(newObj, key, desc);
            } else {
                newObj[key] = obj[key];
            }
        }
    }
    newObj.default = obj;
    if (cache) {
        cache.set(obj, newObj);
    }
    return newObj;
}
const schemaBuilder = _core.default.prototype;
const OrderByRefMap = new WeakMap();
const PrismaStringFilterModeRefMap = new WeakMap();
const nullableOps = new Set([
    'equals',
    'is',
    'isNot',
    'not'
]);
schemaBuilder.prismaFilter = function prismaFilter(type, { ops, name, ...options }) {
    const filterName = name !== null && name !== void 0 ? name : `${nameFromType(type, this)}Filter`;
    const ref = this.inputRef(filterName);
    const opsOptions = Array.isArray(ops) ? ops.reduce((map, op)=>{
        map[op] = {};
        return map;
    }, {}) : ops;
    ref.implement({
        ...options,
        extensions: {
            ...options.extensions,
            pothosPrismaInput: {
                nullableFields: nullableOps
            }
        },
        fields: (t)=>{
            const fields = {};
            for (const op of Object.keys(opsOptions)){
                const isList = op === 'in' || op === 'notIn';
                let fieldType;
                switch(op){
                    case 'not':
                        fieldType = ref;
                        break;
                    case 'in':
                    case 'notIn':
                        fieldType = [
                            type
                        ];
                        break;
                    case 'mode':
                        fieldType = this.prismaStringFilterModeEnum();
                        break;
                    default:
                        fieldType = type;
                }
                fields[op] = t.field({
                    required: isList ? {
                        list: false,
                        items: true
                    } : false,
                    type: fieldType,
                    ...opsOptions[op]
                });
            }
            return fields;
        }
    });
    return ref;
};
schemaBuilder.prismaStringFilterModeEnum = function prismaStringFilterModeEnum() {
    if (PrismaStringFilterModeRefMap.has(this)) {
        return PrismaStringFilterModeRefMap.get(this);
    }
    const ref = this.enumType('StringFilterMode', {
        values: {
            Default: {
                value: 'default'
            },
            Insensitive: {
                value: 'insensitive'
            }
        }
    });
    PrismaStringFilterModeRefMap.set(this, ref);
    return ref;
};
schemaBuilder.prismaListFilter = function prismaListFilter(type, { name, ops, ...options }) {
    let filterName = name;
    if (!filterName) {
        const typeName = nameFromType(type, this);
        filterName = typeName.endsWith('Filter') ? typeName.replace(/Filter$/, 'ListFilter') : `List${typeName}`;
    }
    const ref = this.inputRef(filterName);
    const opsOptions = Array.isArray(ops) ? ops.reduce((map, op)=>{
        map[op] = {};
        return map;
    }, {}) : ops;
    ref.implement({
        ...options,
        extensions: {
            ...options.extensions,
            pothosPrismaInput: true
        },
        fields: (t)=>{
            const fields = {};
            for (const op of Object.keys(opsOptions)){
                fields[op] = t.field({
                    required: false,
                    type,
                    ...opsOptions[op]
                });
            }
            return fields;
        }
    });
    return ref;
};
schemaBuilder.prismaScalarListFilter = function prismaScalarListFilter(type, { name, ops, ...options }) {
    let filterName = name;
    if (!filterName) {
        const typeName = nameFromType(type, this);
        filterName = `${typeName}ListFilter`;
    }
    const ref = this.inputRef(filterName);
    const opsOptions = Array.isArray(ops) ? ops.reduce((map, op)=>{
        map[op] = {};
        return map;
    }, {}) : ops;
    ref.implement({
        ...options,
        extensions: {
            ...options.extensions,
            pothosPrismaInput: true
        },
        fields: (t)=>{
            const fields = {};
            for (const op of Object.keys(opsOptions)){
                let fieldType = type;
                switch(op){
                    case 'has':
                        fieldType = type;
                        break;
                    case 'equals':
                    case 'hasSome':
                    case 'hasEvery':
                        fieldType = [
                            type
                        ];
                        break;
                    case 'isEmpty':
                        fieldType = 'Boolean';
                        break;
                    default:
                        throw new Error(`Invalid op ${op} for scalar list filter`);
                }
                fields[op] = t.field({
                    required: false,
                    type: fieldType,
                    ...opsOptions[op]
                });
            }
            return fields;
        }
    });
    return ref;
};
schemaBuilder.orderByEnum = function orderByEnum() {
    if (OrderByRefMap.has(this)) {
        return OrderByRefMap.get(this);
    }
    const ref = this.enumType('OrderBy', {
        values: {
            Asc: {
                value: 'asc'
            },
            Desc: {
                value: 'desc'
            }
        }
    });
    OrderByRefMap.set(this, ref);
    return ref;
};
schemaBuilder.prismaOrderBy = function prismaOrderBy(type, { name, fields, ...options }) {
    const filterName = name !== null && name !== void 0 ? name : `${nameFromType(type, this)}OrderBy`;
    const ref = this.inputRef(filterName);
    ref.implement({
        ...options,
        extensions: {
            ...options.extensions,
            pothosPrismaInput: true
        },
        fields: (t)=>{
            const fieldDefs = {};
            const fieldMap = typeof fields === 'function' ? fields() : fields;
            for (const [field, fieldOption] of Object.entries(fieldMap)){
                if (typeof fieldOption === 'function') {
                    const { type: fieldType, ...fieldOptions } = fieldOption();
                    fieldDefs[field] = t.field({
                        required: false,
                        ...fieldOptions,
                        type: fieldType
                    });
                } else if (typeof fieldOption === 'boolean') {
                    fieldDefs[field] = t.field({
                        required: false,
                        type: this.orderByEnum()
                    });
                } else {
                    fieldDefs[field] = t.field({
                        required: false,
                        type: fieldOption
                    });
                }
            }
            return fieldDefs;
        }
    });
    return ref;
};
schemaBuilder.prismaWhere = function prismaWhere(type, { name, fields, ...options }) {
    const ref = this.inputRef(name !== null && name !== void 0 ? name : `${nameFromType(type, this)}Filter`);
    const model = (0, _pluginprisma.getModel)(type, this);
    const nullableFields = new Set(model.fields.filter((field)=>!field.isRequired).map((field)=>field.name));
    ref.implement({
        ...options,
        extensions: {
            ...options.extensions,
            pothosPrismaInput: {
                nullableFields
            }
        },
        fields: (t)=>{
            const fieldDefs = {};
            const fieldMap = typeof fields === 'function' ? fields(t) : fields;
            for (const [field, fieldOption] of Object.entries(fieldMap)){
                if (!fieldOption) {
                    continue;
                }
                if (field === 'AND' || field === 'OR' || field === 'NOT') {
                    const fieldType = fieldOption === true ? ref : fieldOption;
                    fieldDefs[field] = fieldOption instanceof _core.InputFieldRef ? fieldOption : t.field({
                        required: false,
                        type: field === 'NOT' ? fieldType : [
                            fieldType
                        ],
                        ...typeof fieldOption === 'object' ? fieldOption : {}
                    });
                    continue;
                }
                fieldDefs[field] = fieldOption instanceof _core.InputFieldRef ? fieldOption : t.field({
                    required: false,
                    type: fieldOption
                });
            }
            return fieldDefs;
        }
    });
    return ref;
};
schemaBuilder.prismaWhereUnique = function prismaWhereUnique(type, { name, fields, ...options }) {
    const ref = this.inputRef(name !== null && name !== void 0 ? name : `${nameFromType(type, this)}UniqueFilter`);
    ref.implement({
        ...options,
        extensions: {
            ...options.extensions,
            pothosPrismaInput: true
        },
        fields: (t)=>{
            const fieldDefs = {};
            const fieldMap = typeof fields === 'function' ? fields(t) : fields;
            for (const [field, fieldOption] of Object.entries(fieldMap)){
                if (!fieldOption) {
                    continue;
                }
                fieldDefs[field] = fieldOption instanceof _core.InputFieldRef ? fieldOption : t.field({
                    required: false,
                    type: fieldOption
                });
            }
            return fieldDefs;
        }
    });
    return ref;
};
schemaBuilder.prismaCreate = function prismaCreate(type, { name, fields, ...options }) {
    const ref = this.inputRef(name !== null && name !== void 0 ? name : `${nameFromType(type, this)}CreateInput`);
    const model = (0, _pluginprisma.getModel)(type, this);
    ref.implement({
        ...options,
        extensions: {
            ...options.extensions,
            pothosPrismaInput: true
        },
        fields: (t)=>{
            const fieldDefs = {};
            const fieldMap = typeof fields === 'function' ? fields(t) : fields;
            for (const field of Object.keys(fieldMap)){
                const fieldModel = model.fields.find(({ name: fieldName })=>fieldName === field);
                const fieldOption = fieldMap[field];
                if (!fieldOption) {
                    continue;
                }
                fieldDefs[field] = fieldOption instanceof _core.InputFieldRef ? fieldOption : t.field({
                    required: fieldModel.isRequired && !fieldModel.isList && !fieldModel.hasDefaultValue && !fieldModel.isUpdatedAt,
                    type: fieldModel.isList && fieldModel.kind !== 'object' ? [
                        fieldOption
                    ] : fieldOption
                });
            }
            return fieldDefs;
        }
    });
    return ref;
};
schemaBuilder.prismaCreateMany = function prismaCreateMany(type, { name, fields, ...options }) {
    const ref = this.inputRef(name !== null && name !== void 0 ? name : `${nameFromType(type, this)}CreateManyInput`);
    const model = (0, _pluginprisma.getModel)(type, this);
    ref.implement({
        ...options,
        extensions: {
            ...options.extensions,
            pothosPrismaInput: true
        },
        fields: (t)=>{
            const fieldDefs = {};
            const fieldMap = typeof fields === 'function' ? fields(t) : fields;
            for (const field of Object.keys(fieldMap)){
                const fieldModel = model.fields.find(({ name: fieldName })=>fieldName === field);
                const fieldOption = fieldMap[field];
                if (!fieldOption) {
                    continue;
                }
                fieldDefs[field] = fieldOption instanceof _core.InputFieldRef ? fieldOption : t.field({
                    required: fieldModel.isRequired && !fieldModel.isList && !fieldModel.hasDefaultValue && !fieldModel.isUpdatedAt,
                    type: fieldModel.isList && fieldModel.kind !== 'object' ? [
                        fieldOption
                    ] : fieldOption
                });
            }
            return fieldDefs;
        }
    });
    return ref;
};
schemaBuilder.prismaUpdate = function prismaUpdate(type, { name, fields, ...options }) {
    const ref = this.inputRef(name !== null && name !== void 0 ? name : `${nameFromType(type, this)}UpdateInput`);
    const model = (0, _pluginprisma.getModel)(type, this);
    const nullableFields = new Set(model.fields.filter((field)=>!field.isRequired).map((field)=>field.name));
    ref.implement({
        ...options,
        extensions: {
            ...options.extensions,
            pothosPrismaInput: {
                nullableFields
            }
        },
        fields: (t)=>{
            const fieldDefs = {};
            const fieldMap = typeof fields === 'function' ? fields(t) : fields;
            for (const field of Object.keys(fieldMap)){
                const fieldModel = model.fields.find(({ name: fieldName })=>fieldName === field);
                const fieldOption = fieldMap[field];
                if (!fieldOption) {
                    continue;
                }
                fieldDefs[field] = fieldOption instanceof _core.InputFieldRef ? fieldOption : t.field({
                    required: false,
                    type: fieldModel.isList && fieldModel.kind !== 'object' ? [
                        fieldOption
                    ] : fieldOption
                });
            }
            return fieldDefs;
        }
    });
    return ref;
};
schemaBuilder.prismaCreateRelation = function prismaCreateRelation(type, relation, { name, fields, ...options }) {
    const ref = this.inputRef(name !== null && name !== void 0 ? name : `${nameFromType(type, this)}Create${capitalize(relation)}RelationInput`);
    const model = (0, _pluginprisma.getModel)(type, this);
    const fieldModel = model.fields.find((field)=>field.name === relation);
    ref.implement({
        ...options,
        extensions: {
            ...options.extensions,
            pothosPrismaInput: true
        },
        fields: (t)=>{
            const fieldDefs = {};
            const fieldMap = typeof fields === 'function' ? fields(t) : fields;
            for (const [field, fieldOption] of Object.entries(fieldMap)){
                if (!fieldOption) {
                    continue;
                }
                fieldDefs[field] = fieldOption instanceof _core.InputFieldRef ? fieldOption : t.field({
                    required: false,
                    type: fieldModel.isList ? [
                        fieldOption
                    ] : fieldOption
                });
            }
            return fieldDefs;
        }
    });
    return ref;
};
schemaBuilder.prismaUpdateRelation = function prismaUpdateRelation(type, relation, { name, fields, ...options }) {
    const ref = this.inputRef(name !== null && name !== void 0 ? name : `${nameFromType(type, this)}Update${capitalize(relation)}RelationInput`);
    ref.implement({
        ...options,
        extensions: {
            ...options.extensions,
            pothosPrismaInput: true
        },
        fields: (t)=>{
            const fieldDefs = {};
            const fieldMap = typeof fields === 'function' ? fields(t) : fields;
            const model = (0, _pluginprisma.getModel)(type, this);
            const fieldModel = model.fields.find((field)=>field.name === relation);
            for (const [field, fieldOption] of Object.entries(fieldMap)){
                if (!fieldOption) {
                    continue;
                }
                if (fieldOption instanceof _core.InputFieldRef) {
                    fieldDefs[field] = fieldOption;
                } else if (fieldModel.isList && (field === 'update' || field === 'updateMany')) {
                    const { name: nestedName = `${ref.name}U${field.slice(1)}`, where: whereType, data: dataType } = fieldOption;
                    const nestedRef = this.inputType(nestedName, {
                        fields: (t2)=>({
                                where: whereType instanceof _core.InputFieldRef ? whereType : t2.field({
                                    type: whereType
                                }),
                                data: dataType instanceof _core.InputFieldRef ? dataType : t2.field({
                                    type: dataType
                                })
                            })
                    });
                    fieldDefs[field] = t.field({
                        required: false,
                        type: [
                            nestedRef
                        ]
                    });
                } else if (fieldModel.isList && field === 'createMany') {
                    const { name: nestedName = `${ref.name}CreateMany`, data: dataType, skipDuplicates } = fieldOption;
                    const nestedRef = this.inputType(nestedName, {
                        fields: (t2)=>({
                                ...skipDuplicates ? {
                                    skipDuplicates: skipDuplicates instanceof _core.InputFieldRef ? skipDuplicates : t2.field({
                                        type: skipDuplicates
                                    })
                                } : {},
                                data: dataType instanceof _core.InputFieldRef ? dataType : t2.field({
                                    type: [
                                        dataType
                                    ]
                                })
                            })
                    });
                    fieldDefs[field] = t.field({
                        required: false,
                        type: nestedRef
                    });
                } else if ((field === 'disconnect' || field === 'delete') && fieldOption === true) {
                    fieldDefs[field] = t.field({
                        required: false,
                        type: 'Boolean'
                    });
                } else {
                    fieldDefs[field] = t.field({
                        required: false,
                        type: fieldModel.isList ? [
                            fieldOption
                        ] : fieldOption
                    });
                }
            }
            return fieldDefs;
        }
    });
    return ref;
};
function nameFromType(type, builder) {
    if (typeof type === 'string') {
        return type;
    }
    if (builder.configStore.hasConfig(type)) {
        return builder.configStore.getTypeConfig(type).name;
    }
    if (typeof type === 'function' && 'name' in type) {
        return type.name;
    }
    if (type instanceof _core.BaseTypeRef) {
        return type.name;
    }
    throw new _core.PothosSchemaError(`Unable to determine name for type ${String(type)}`);
}
schemaBuilder.prismaIntAtomicUpdate = function prismaIntUpdateOperations({ name, ops = [
    'set',
    'increment',
    'decrement',
    'multiply',
    'divide'
], ...options } = {}) {
    const ref = this.inputRef(name !== null && name !== void 0 ? name : 'IntAtomicUpdate');
    ref.implement({
        ...options,
        extensions: {
            ...options.extensions,
            pothosPrismaInput: true
        },
        fields: (t)=>{
            const fieldDefs = {};
            for (const op of ops){
                fieldDefs[op] = t.field({
                    required: false,
                    type: 'Int'
                });
            }
            return fieldDefs;
        }
    });
    return ref;
};
function capitalize(str) {
    return str[0].toUpperCase() + str.slice(1);
}

//# sourceMappingURL=schema-builder.js.map